/*  	ccide - C Language Decision Table Code Generator 
	Copyright (C) 2002-2004,2010,2012;  Thomas W. Young, e-mail:  ccide@twyoung.com

    	This file is part of ccide, the C Language Decision Table Code Generator.

   	Ccide is free software: you can redistribute it and/or modify
   	it under the terms of the GNU General Public License as published by
    	the Free Software Foundation, either version 3 of the License, or
   	(at your option) any later version.

    	Ccide is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with Ccide.  If not, see <http://www.gnu.org/licenses/> or
    	write to the Free Software Foundation, Inc., 51 Franklin St, 
    	Fifth Floor, Boston, MA 02110-1301 USA.
*/

/* THISIS: if2rpntok.c.d 7/19/2012 */

#include <stdio.h>     
#include <string.h>
#include "if2rpn.h"

#define SIZE 1000 
#define S(C,T) table[C] = T

#ifdef UNIT_TEST 
TOKEN token;
char *tname[NTOK]={"EMPTY","NUMBER","FUNCTION_TOKEN","SEPARATOR","LEFT_PAREN",
		"RIGHT_PAREN","OPERATOR","EQUAL","PLUS","MINUS",
		"TIMES","DIV"};
char *ttable[]={".", "n", "f", "|", "(", ")", "o", "=", "+", "-", "*", "/"};  /* Tokens */
#endif

TOKEN_TYPE table[256];
TOKEN_TYPE type;
int number;

static char *ch=NULL,bfr[SIZE+1];
static FILE *in;

//CCIDE_INLINE_CODE:
/*GENERATED_CODE: */
#ifndef __CCIDE_INLINE_C
#define __CCIDE_INLINE_C

/*ccide-0.6.6-1
ccide is Copyright (C) 2002-2004,2010,2012;  Thomas W. Young, e-mail:  ccide@twyoung.com 
The code generated by ccide is covered by the same license as the source  
code(decision table) from which it is derived. If you created the source,  
you are free to do anything you like with the generated code, 
including incorporating it into or linking it with proprietary software.  
*/ 
static int ccide_group=1; 
#ifndef UINT_MAX 
#include "limits.h" 
#endif /* End if not defined UINT_MAX */ 
 
		/* Return rule number */ 
static int CCIDEFindRule(
	int nbrrules,  unsigned long ccide_table, unsigned long yes[], unsigned long no[]){ 
        int r=0; 
        unsigned long nstate; 
 
        nstate = UINT_MAX ^ ccide_table; 
 
        while ( 
		( (yes[r] & nstate) || (no[r]  & ccide_table) ) 
		 && ( ++r < nbrrules )  
	) {}; 
 
        return r; 
} 
 
static int CCIDEFindRuleYes(             /* Return rule number */
	int nbrrules, unsigned long ccide_table, unsigned long yes[]) 
{ 
        int r=0; 
        unsigned long nstate;
 
        nstate = UINT_MAX ^ ccide_table; 
        while ( (yes[r] & nstate) && ( ++r < nbrrules ) ) {}; 
        return r; 
} 
#endif /* End ifndef  __CCIDE_INLINE_C  */
/*END_GENERATED_CODE: */

static TOKEN_TYPE Lookup(char c) {
	return table[c]; 
}

static void Setup(){
	int i;

	for(i=0;i<10;i++) {
		S(i+'0',NUMBER);
	}
	S(';',SEPARATOR);
	S('+',PLUS);
	S('-',MINUS);
	S('=',EQUAL);
	S('*',TIMES);
	S('/',DIV);
	S('(',LEFT_PAREN);
	S(')',RIGHT_PAREN);
}

static TYPE GetType() {

	//DECISION_TABLE:
	//   Y  N | ch==NULL
	// ________________________
	//   X  - | token = SEPARATOR;
	//   X  - | return  SEPARATOR;
	//   -  X | type = Lookup(*ch);
	//END_TABLE:
	//GENERATED_CODE: FOR TABLE_1.
	//	2 Rules, 1 conditions, and 3 actions.
	if( (ch==NULL) )  {
	   token = SEPARATOR;
	   return  SEPARATOR;
	} else { 
	   type = Lookup(*ch);
	}
	//END_GENERATED_CODE: FOR TABLE_1, by ccide-0.6.6-1  


	//DECISION_TABLE:
	//   Y  N  N | type == NUMBER
	//   -  Y  N | type > OPERATOR
	// __________________________
	//   X  -  - | number = atoi(ch);
	//   X  -  - | return   NUMBER;
	//   -  X  X | token  = type;
	//   -  X  - | return   OPERATOR;
	//   -  -  X | return   token;
	//END_TABLE:
	//GENERATED_CODE: FOR TABLE_2.
	//	3 Rules, 2 conditions, and 5 actions.
	 {	unsigned long CCIDE_table2_yes[3]={   2UL,   0UL,   1UL};
		unsigned long CCIDE_table2_no[3]= {   1UL,   3UL,   0UL};


		switch(CCIDEFindRule(3,
			  (type == NUMBER)
			| (type > OPERATOR)<<1
			  ,CCIDE_table2_yes, CCIDE_table2_no)) {
		case  1:	//	Rule  3 
		    token  = type;
		    return   token;
		    break;
		case  0:	//	Rule  2 
		    token  = type;
		    return   OPERATOR;
		    break;
		case  2:	//	Rule  1 
		    number = atoi(ch);
		    return   NUMBER;
		    break;
		} // End Switch
	}
	//END_GENERATED_CODE: FOR TABLE_2, by ccide-0.6.6-1  

}

#ifndef PSEUDO_INPUT
TYPE GetToken() {
	static STATE s=2;

	//DECISION_TABLE:
	//   2  3  0  1 | s==$$
	// _____________________
	//   X  -  -  - | Setup();
	//   -  -  -  - | s = (  (in = fopen("./if2rpn.txt","r"))== NULL );
	//   0  -  -  - | s = $$;
	//   -  -  -  - | perror("Opening infix.txt"); return 1;
	//   X  -  -  - | ch=NULL;
	//   -  -  -  - | goto $@;
	//END_TABLE:
	//GENERATED_CODE: FOR TABLE_3.
	//	1 Rules, 4 conditions, and 6 actions.

	CCIDE_TABLE_3:	if( (s==2) )  {
	   Setup();
	   s = 0;
	   ch=NULL;
	}
	//END_GENERATED_CODE: FOR TABLE_3, by ccide-0.6.6-1  

    {
	static STATE s2=2;	
	//DECISION_TABLE:
	//   4  2  0  -  3  3 | s2==$$
	//   -  -  -  -  N  Y | ch==NULL
	// ___________________________
	//   -  X  -  -  -  - | s2=(fgets(bfr,SIZE,stdin) == NULL);
	//   -  -  X  -  -  - | ch = strtok(bfr, " \n");
	//   X  -  -  -  -  - | ch = strtok(NULL, " \n");
	//   3  -  3  -  4  2 | s2 = $$; 
	//   -  -  -  -  -  - | ch=NULL;
	//   -  -  -  -  X  X | return GetType();
	//   -  -  -  -  -  - | return SEPARATOR;
	//   -  -  -  X  -  - | return EMPTY;
	//   X  X  X  -  -  - | goto $@;
	//END_TABLE:
	//GENERATED_CODE: FOR TABLE_4.
	//	6 Rules, 5 conditions, and 11 actions.
	 {	unsigned long CCIDE_table4_yes[6]={   8UL,  24UL,   1UL,   2UL,   4UL,   0UL};
		unsigned long CCIDE_table4_no[6]= {  16UL,   0UL,   0UL,   0UL,   0UL,   0UL};

	CCIDE_TABLE_4:
		switch(CCIDEFindRule(6,
			  (s2==4)
			| (s2==2)<<1
			| (s2==0)<<2
			| (s2==3)<<3
			| (ch==NULL)<<4
			  ,CCIDE_table4_yes, CCIDE_table4_no)) {
		case  2:	//	Rule  1 
		    ch = strtok(NULL, " \n");
		    s2 = 3;
		    goto CCIDE_TABLE_4 ;
		case  4:	//	Rule  3 
		    ch = strtok(bfr, " \n");
		    s2 = 3;
		    goto CCIDE_TABLE_4 ;
		case  3:	//	Rule  2 
		    s2=(fgets(bfr,SIZE,stdin) == NULL);
		    goto CCIDE_TABLE_4 ;
		case  5:	//	Rule  4 
		    return EMPTY;
		    break;
		case  1:	//	Rule  6 
		    s2 = 2;
		    return GetType();
		    break;
		case  0:	//	Rule  5 
		    s2 = 4;
		    return GetType();
		    break;
		} // End Switch
	}
	//END_GENERATED_CODE: FOR TABLE_4, by ccide-0.6.6-1  

   }

   // return EMPTY;  // Uncomment to avoid possible compiler error message.
}
#endif

#ifdef UNIT_TEST
int number;
int main() {
	TYPE token=9;
	STATE s=0;

	//DECISION_TABLE:
	//   0  1  1  - | s == $$
	//   -  Y  Y  - | token != EMPTY
	//   -  Y  -  - | token == NUMBER
	// _____________________________
	//   -  X  -  - | printf("%i %s\n",number, tname[t]);
	//   -  -  X  - | printf("%s %s\n",ttable[token], tname[t]);
	//   1  -  -  - | s=$$;
	//   X  X  X  - | token = GetToken();
	//   X  X  X  - | goto $@;
	//END_TABLE:
	//GENERATED_CODE: FOR TABLE_5.
	//	3 Rules, 4 conditions, and 5 actions.
	 {	unsigned long CCIDE_table5_yes[3]={  14UL,   6UL,   1UL};

	CCIDE_TABLE_5:
		switch(CCIDEFindRuleYes(3,
			  (s == 0)
			| (s == 1)<<1
			| (token != EMPTY)<<2
			| (token == NUMBER)<<3
			  ,CCIDE_table5_yes)) {
		case  2:	//	Rule  1 
		    s=1;
		    token = GetToken();
		    goto CCIDE_TABLE_5 ;
		case  1:	//	Rule  3 
		    printf("%s %s\n",ttable[token], tname[t]);
		    token = GetToken();
		    goto CCIDE_TABLE_5 ;
		case  0:	//	Rule  2 
		    printf("%i %s\n",number, tname[t]);
		    token = GetToken();
		    goto CCIDE_TABLE_5 ;
		} // End Switch
	}
	//END_GENERATED_CODE: FOR TABLE_5, by ccide-0.6.6-1  

}

#endif
